/*
 * @Copyright   Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module      ckeditor/plugins/epgadget/plugin
 *
 * @revision    $$
 */
/*global define, window, de_epages*/
define('ckeditor/plugins/epgadget/plugin', [
    'jquery',
    'ep',
    'ckeditor/ckeditor',
    'util/json',
    'de_epages/externalcontent/gadgetdialog/constants',
    '$dict!ep/dict'
], function ($, ep, CKEDITOR, JSON, CONSTANTS, epDict) {
    'use strict';

    var selectedNode,
        tGadget = epDict.translate('Gadget'),
        topRequire = window.require,
        gadgetClass = CONSTANTS.gClassname,
        pluginName = 'epgadget',
        // get the correct image source for each gadget
        createImgSrc = function (data) {
            var srcValue = ep.config.iconsRoot + '/gadget',
                opt;

            if (data) {
                srcValue += '_' + data.alias;
                opt = data.options;
                if (opt && data.alias === 'maps') {
                    if (opt.layout) {
                        srcValue += '_' + opt.layout;
                    }
                    if (opt.type) {
                        srcValue += '_' + opt.type;
                    }
                }
            }

            srcValue += '.png';
            return srcValue;
        }

    // Work-around for opening dialog outside of iframe,
    // when CKEditor is instanciated on the SF inside an iframe in MBO.
    if (window.top !== window.self) {
        try {
            topRequire = window.top.require;
        } catch (ignore) {}
    }

    function unprotectRealComments( html )
    {
        return html.replace( /<!--\{cke_protected\}\{C\}([\s\S]+?)-->/g, function( match, data )
            {
                return decodeURIComponent( data );
            });
    }

    function unprotectSource( html, editor )
    {
        var store = editor._.dataStore;

        return html.replace( /<!--\{cke_protected\}([\s\S]+?)-->/g, function( match, data )
            {
                return decodeURIComponent( data );
            }).replace( /\{cke_protected_(\d+)\}/g, function( match, id )
            {
                return store && store[ id ] || '';
            });
    }

    var baseToDataFormat = CKEDITOR.htmlDataProcessor.prototype.toDataFormat;
    var beforeToDataFormat = function(html, fixForBody) {
        var fragment = $('<div />').html(html),
            i = 0;

        fragment.find('img.ep-gadget').addClass('epValidationNotDone');
        while (fragment.find('.epValidationNotDone').length) {
            validate(fragment.find('.epValidationNotDone:first'), fragment);
            i++;
        }

        fragment.find('p').each(function (i, elem) {
            var $elem = $(elem);
            if ($elem.text().replace(/\s|\n|\r/g, '').charCodeAt(0) === 8203 && $elem.text().length === 1) {
                $elem.remove();
            }
        });

        return fragment.html();
    }

    CKEDITOR.htmlDataProcessor.prototype.toDataFormat = function(html, fixForBody){
        if (beforeToDataFormat) {
            // html = beforeToDataFormat(html, fixForBody);
            html = beforeToDataFormat(html, fixForBody);
        }

        var writer = this.writer,
            fragment = CKEDITOR.htmlParser.fragment.fromHtml( html, fixForBody );

        writer.reset();

        fragment.writeHtml( writer, this.htmlFilter );

        var data = writer.getHtml( true );

        // Restore those non-HTML protected source. (#4475,#4880)
        data = unprotectRealComments( data );
        data = unprotectSource( data, this.editor );

        return data;
    }

    function validate(element, fragment) {
        var tmpid = 'removeme_' + new Date().getTime();
        element
            .attr({
                'id': tmpid,
                'style': (element.attr('style') ? element.attr('style').replace(/\s*?float:.*?;/g, '') : '')
            })
            .removeClass('epValidationNotDone');

        var parentNode = element.parent(),
            i = 0,
            invalidParentStack = [],
            validParentContent,
            elementContent,
            closeTags = '',
            openTags = '',
            arrayLength,
            validElementContent = '',
            necessaryAttributes = ['class', 'style', 'id'],
            tmpContentDiv = $('<div />');

        // iterate over not allowed parent nodes
        while (i < 100 && parentNode[0]) {
            // push invalid element to stack
            invalidParentStack.push(parentNode);

            // check if gadget element is allowed in current parent node
            if (CKEDITOR.dtd[parentNode[0].nodeName.toLowerCase()]['div']) {
                // if allowed -> remove last element from invalid stack
                invalidParentStack.pop();
                break;
            }

            // add class to element (is needed later to remove empty elements from the DOM)
            parentNode.addClass('ep-removeThisElementIfEmpty');
            // get the next parent node
            parentNode = parentNode.parent();

            i++;
        }

        // get the inner HTML string from the parent node
        validParentContent = parentNode.html();
        // get the HTML string from the gadget element

        elementContent = element[0].outerHTML;

        i = 0;
        arrayLength = invalidParentStack.length - 1;

        // iterate over all invalid elements
        for (i; i <= arrayLength; i++) {
            var attribute,
                attributes = [],
                attributesString = '';

            // iterate over all relevant attributes
            for (attribute in necessaryAttributes) {
                // if the current attribute is defined in the current invalid element
                if (invalidParentStack[arrayLength - i].attr(necessaryAttributes[attribute])) {
                    // concatinate the attribute to the attributes string
                    attributesString += " " + necessaryAttributes[attribute] + "=\"" + invalidParentStack[arrayLength - i].attr(necessaryAttributes[attribute]) + "\"";
                }
            }

            // build the string to complete the tags that have to be opened after the gadget element string
            openTags += '<' + invalidParentStack[arrayLength - i][0].nodeName.toLowerCase() + attributesString + '>';
            // build the string to complete the tags that have to be closed before the gadget element string
            closeTags += '</' + invalidParentStack[i][0].nodeName.toLowerCase() + '>';
        }

        // generate the new inner HTML string
        validElementContent = closeTags + '<p>' + elementContent +'</p>' + openTags;

        // replace the old gadget string with the new content
        validParentContent = validParentContent.replace(elementContent, validElementContent);
        // create HTML code fragment from the new content
        tmpContentDiv.html(validParentContent);
        // loop over the maximum possibilities of empty elements

        for (i = 0; i <= arrayLength; i++) {
            // remove empty element
            tmpContentDiv.find('.ep-removeThisElementIfEmpty').each(function (i, elem) {
                var $elem = $(elem);
                if ($elem.children('br').length === 1 && ($elem.text().replace(/\s|\n|\r|&#8203;/g, '').length === 1 && $elem.text().replace(/\s|\n|\r|&#8203;/g, '').charCodeAt(0) === 8203 || $elem.text().replace(/\s|\n|\r|&#8203;/g, '').length === 0) || $elem.is(':empty')) {
                    $elem.remove();
                }
            })
        }

        // remove class that was added to decide if element was inserted by this script
        tmpContentDiv.find('.ep-removeThisElementIfEmpty').removeClass('ep-removeThisElementIfEmpty');
        var tmpElement = tmpContentDiv.find('#' + tmpid),
            parent = tmpElement.parent();

        tmpElement.removeAttr('id');
        if (parent.text().length > 0) {
            validParentContent = parent.html().replace(tmpElement[0].outerHTML, "</" + parent[0].nodeName.toLowerCase() + ">" + tmpElement[0].outerHTML + "<" + parent[0].nodeName.toLowerCase() + ">");
            parent.html(validParentContent);
            parent.children().each(function (index, elem) {
                if (elem.is(':empty')) {
                    elem.remove();
                }
            });
        }

        // write new content to parent node
        return parentNode.html(tmpContentDiv.html());
    }

    function insertGadget(editor, element, imgSrc) {
        editor.focus();

        var parentNode,
            i = 0,
            invalidParentStack = [],
            validParentContent,
            elementContent,
            closeTags = '',
            openTags = '',
            arrayLength,
            validElementContent = '',
            necessaryAttributes = ['class', 'style', 'id'],
            tmpContentDiv = $('<div />');

        // insert gadget element
        editor.insertElement(element);
        // get parent node
        parentNode = element.getParent();

        // iterate over not allowed parent nodes
        while (i < 100 && parentNode) {
            // push invalid element to stack
            invalidParentStack.push(parentNode);

            // check if gadget element is allowed in current parent node
            if (CKEDITOR.dtd[parentNode.getName().toLowerCase()]['div']) {
                // if allowed -> remove last element from invalid stack
                invalidParentStack.pop();
                break;
            }

            // add class to element (is needed later to remove empty elements from the DOM)
            parentNode.addClass('ep-removeThisElementIfEmpty');
            // get the next parent node
            parentNode = parentNode.getParent();

            i++;
        }

        // get the inner HTML string from the parent node
        validParentContent = parentNode.getHtml();
        // get the HTML string from the gadget element

        elementContent = element.getOuterHtml();
        i = 0;
        arrayLength = invalidParentStack.length - 1;

        // iterate over all invalid elements
        for (i; i <= arrayLength; i++) {
            var attribute,
                attributes = [],
                attributesString = '';

            // iterate over all relevant attributes
            for (attribute in necessaryAttributes) {
                // if the current attribute is defined in the current invalid element
                if (invalidParentStack[arrayLength - i].getAttribute(necessaryAttributes[attribute])) {
                    // concatinate the attribute to the attributes string
                    attributesString += " " + necessaryAttributes[attribute] + "=\"" + invalidParentStack[arrayLength - i].getAttribute(necessaryAttributes[attribute]) + "\"";
                }
            }

            // build the string to complete the tags that have to be opened after the gadget element string
            openTags += '<' + invalidParentStack[arrayLength - i].getName() + attributesString + '>';
            // build the string to complete the tags that have to be closed before the gadget element string
            closeTags += '</' + invalidParentStack[i].getName() + '>';
        }

        // generate the new inner HTML string
        validElementContent = closeTags + '<p>' + elementContent +'</p><p>&#8203;</p>' + openTags;
        // escape special characters
        elementContent = escapeRegExp(elementContent);
        // replace the old gadget string with the new content
        validParentContent = validParentContent.replace(new RegExp(elementContent), validElementContent);

        // create HTML code fragment from the new content
        tmpContentDiv.html(validParentContent);
        // loop over the maximum possibilities of empty elements
        for (i = 0; i <= arrayLength; i++) {
            // remove empty element
            tmpContentDiv.find('.ep-removeThisElementIfEmpty').each(function (i, elem) {
                var $elem = $(elem),
                    innerText = $elem.text(),
                    replacedText = innerText.replace(/\s|\n|\r/g, '');

                // remove all budenlouis
                replacedText = replacedText.replace(new RegExp(String.fromCharCode(8203), "g"), "");

                if (($elem.children('br').length === 1 && $elem.children().length===1) || (replacedText.length === 0 && $elem.children().length===0) || $elem.is(':empty')) {
                    $elem.remove();
                }
            })
        }

        // remove class that was added to decide if element was inserted by this script
        tmpContentDiv.find('.ep-removeThisElementIfEmpty').removeClass('ep-removeThisElementIfEmpty');
        tmpContentDiv.find('#' + element.getAttribute('id')).css({
            'background': 'url(' + imgSrc + ') 0 0 no-repeat'
        });

        // write new content to parent node
        return parentNode.setHtml(tmpContentDiv.html());
    }

    function escapeRegExp(str) {
        return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
    }

    function setCaret(editor, id) {
        // contentWindow: Get window object of editor iframe
        // doc: get the document object
        // el: get editable element
        var contentWindow = $(editor.container.$).find('iframe')[0].contentWindow,
            doc = $(editor.container.$).find('iframe').contents()[0],
            el = doc.activeElement;

        // range: new range on document
        // sel: get current selection
        // gadget: inserted gadget
        // nextElement: element where to set cursor
        var range = doc.createRange(),
            sel = contentWindow.getSelection(),
            gadget = doc.getElementById(id),
            nextElement = gadget.parentNode.nextSibling;

        // remove previously added gadget identifier
        gadget.removeAttribute('id');
        // set cursor at the beginning of nextElement
        range.setStart(nextElement, 0);
        // set start and end to the same position
        range.collapse(true);
        // remove all existing ranges
        sel.removeAllRanges();
        // add new selection
        sel.addRange(range);

        // set focus
        window.setTimeout(function () {
            el.focus();
        });
    }

    CKEDITOR.plugins.add(pluginName, {

        init: function (editor) {

            editor.addCommand(pluginName, {
                exec: function (editor, alias) {
                    var selectedElement, dataAttribute, gadgetOptions = {
                        // callback for de_epages/externalcontent/gadgetdialog/main-view.js (close)
                        onDialogClose: function () {
                            // set selected node to undefined if dialog is closed
                            selectedNode = undefined;
                        },

                        // callback for de_epages/externalcontent/gadgetdialog/main-view.js (apply)
                        callback: function (data) {
                            var imgSrc = createImgSrc(data),
                                // gadget element (take existing if there is already one, otherwise create a new one)
                                element = selectedNode ? selectedNode : CKEDITOR.dom.element.createFromHtml( '<img/>' ),
                                dataString = JSON.stringify(data),
                                id = 'insertNewGadget_' + new Date().getTime();

                             // set/update attributes and style
                            element
                                .setAttributes({
                                    'alt': tGadget,
                                    'title': tGadget,
                                    'class': gadgetClass,
                                    'data-ep-gadget': dataString,
                                    'src': 'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR4nGP6zwAAAgcBApocMXEAAAAASUVORK5CYII=',
                                    'id': id
                                })
                                .setStyles({
                                    'max-width': '100%',
                                    'display': 'block',
                                    'margin-bottom': '10px',
                                    'width': '100%',
                                    'height': '111px'
                                });
                            // if no element is selected -> insert new gadget element and set cursor
                            if (!selectedNode) {
                                insertGadget(editor, element, imgSrc);
                                setCaret(editor, id);
                            } else {
                                element.setStyles({
                                    'background': 'url(' + imgSrc + ') 0 0 no-repeat'
                                });
                            }
                        }
                    };

                    // Check if gadget is selected and get gadget data from *fakeElement*.
                    if (!selectedNode) {
                        var currentSelection = editor.getSelection();
                        if (currentSelection) {
                            selectedNode = currentSelection.getSelectedElement();
                            if (selectedNode) {
                                dataAttribute = selectedNode.getAttribute('data-ep-gadget');
                                if (dataAttribute) {
                                    gadgetOptions.gadget = JSON.parse(dataAttribute);
                                }
                            }
                        }
                    } else {
                        dataAttribute = selectedNode.getAttribute('data-ep-gadget');
                        if (dataAttribute) {
                            gadgetOptions.gadget = JSON.parse(dataAttribute);
                        }
                    }

                    if (alias && alias === 'rss' && !gadgetOptions.gadget) {
                        gadgetOptions.gadget = {
                            alias: 'rss'
                        };
                    }

                    topRequire(['de_epages/externalcontent/gadgetdialog'], function (gadgetsdialog) {
                        // with EPG-28229 we removed the types "profile" and "search"
                        // but it is possible that merchants still have these types in their shop
                        // so we will send them to the standard twitter gadget, when they edit the old profile/search-twitter gadget.
                        if (gadgetOptions.gadget && gadgetOptions.gadget.alias === 'twitter' && (gadgetOptions.gadget.options.type === 'profile' || gadgetOptions.gadget.options.type === 'search')) {
                            gadgetOptions.gadget.options = {};
                        }
                        gadgetsdialog.open(gadgetOptions);
                    });
                }
            });

            editor.ui.addButton(pluginName, {
                label: tGadget,
                command: pluginName,
                icon: this.path + 'images/gadget.png',
                onRender: function () {
                    // Highlight gadget button on gadget selection.
                    editor.on('selectionChange', function (ev) {
                        this.setState(CKEDITOR.TRISTATE_OFF);

                        if (ev.data && ev.data.selection && ev.data.selection.getSelectedElement() !== null && ev.data.selection.getSelectedElement().hasClass(gadgetClass)) {
                            this.setState(CKEDITOR.TRISTATE_ON);
                        }
                    }, this);
                }
            });

            // after editor was built
            editor.on('instanceReady', function (ev) {
                // handle click on editor
                editor.on('click', function (ev) {
                    // if a gadget element was clicked
                    if (ev.data && ev.data.element && ev.data.element.hasClass(gadgetClass)) {
                        // set variable to the clicked node
                        selectedNode = ev.data.element;
                    } else {
                        // "remove" selection
                        selectedNode = undefined;
                    }
                });
            });

            editor.on('doubleclick', function (ev) {
                if (ev.data && ev.data.element && ev.data.element.hasClass(gadgetClass)) {
                    selectedNode = ev.data.element;
                    editor.execCommand(pluginName, ev.data.element);
                }
            });
        },

        afterInit: function (editor) {
            var dataProcessor = editor.dataProcessor,
                dataFilter = dataProcessor && dataProcessor.dataFilter,
                htmlFilter = dataProcessor && dataProcessor.htmlFilter;

            if (dataFilter) {
                dataFilter.addRules({
                    elements: {
                        div: function (element) {
                            // Select gadget and replace it with *fakeElement*.
                            var classes = element.attributes['class'];
                            if (/\bep-gadget\b/.test(classes)) {
                                var dataString = $('<div/>').html(element.attributes['data-ep-gadget']).text(),
                                    data = JSON.parse(dataString),
                                    imgSrc = createImgSrc(data),
                                    newHtmlParserElementP = new CKEDITOR.htmlParser.element('p'),
                                    newHtmlParserElementIMG = new CKEDITOR.htmlParser.element('img', {
                                        'alt': tGadget,
                                        'title': tGadget,
                                        'class': classes,
                                        'data-ep-gadget': dataString,
                                        'src': 'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR4nGP6zwAAAgcBApocMXEAAAAASUVORK5CYII=',
                                        'style': (element.attributes['style'] ? element.attributes['style'] : '') + 'max-width: 100%; display: block; margin-bottom: 10px; width: 100%; height: 111px; background: url(' + imgSrc + ') 0 0 no-repeat;',
                                        'id': element.attributes['id']
                                    });
                                newHtmlParserElementP.add(newHtmlParserElementIMG);
                                return newHtmlParserElementP;
                            }
                        },
                        p: function (element) {
                            var child = element.children[0],
                                pattern = /\bep-gadget\b/;      // regular expression to indicate old gadgets with DIV by class name

                            // is p tag empty?
                            if (child && !child.children && (!child.value || CKEDITOR.tools.trim(child.value).match(/^(?:&nbsp;|\xa0|<br \/>)$/))) {
                                // is previous or next element a gadget div -> remove empty p tag
                                if ((element.previous && element.previous.attributes && element.next && pattern.test(element.previous.attributes['class'])) || (element.next && element.next.attributes && pattern.test(element.next.attributes['class']))) {
                                    return false;
                                }
                            }

                            return element;
                        }
                    }
                }, 100);
            }
        }
    });

    return CKEDITOR;
});