/*jslint nomen: true*/
/*global define, require*/
define('de_epages/externalcontent/gadgetdialog/main-view', [
    'jquery',
    'underscore',
    'backbone',
    'ep/ui/dialog',
    '$dict!../dictionary'
], function ($, _, Backbone, ep, dict) {

    'use strict';

    var modulePath = 'de_epages/externalcontent/gadgetdialog/',

        MainView = Backbone.View.extend({

            tagName: 'div',

            render: function () {
                if (this.content) {
                    this.$el.append(this.content.render().$el);
                }
                return this;
            },

            close: function () {
                if (this.content) {
                    this.content.remove();
                    this.content = null;
                }
                this.dialogNode.uiDialog('close');
            },

            open: function (options) {
                var $this = this;

                this.options = options;

                // if a callback function was handed over to do something before closing the gadget dialog
                if ($this.options.onDialogClose) {
                    // set dialog option
                    this.dialogNode.uiDialog('option', {
                        beforeClose: function () {
                            if ($this.options.onDialogClose) {
                                // execute callback function
                                $this.options.onDialogClose();
                            }
                        }
                    });
                }

                this.updateContentView();
            },

            apply: function () {
                // *this.content.onApply* can still stop the process.
                var doApply = true;

                if (this.content && typeof this.content.onApply === 'function') {
                    doApply = this.content.onApply();
                }

                if (doApply) {
                    if (this.options.callback) {
                        this.options.callback({
                            'options': this.content.gOptions,
                            'alias': this.content.alias,
                            'cartridge': 'externalcontent'
                        });
                    }
                    this.close();
                }

            },

            removeContent: function () {
                // Out with the old content.
                if (this.content) {
                    // Remove content *Backbone.View* from the DOM,
                    // and call stopListening to remove any bound
                    // events that the view has listenTo'd.
                    this.content.remove();
                    // Remove all of the view's delegated events.
                    this.content.undelegateEvents();
                    this.content = null;
                }
            },

            renderWith: function (contentView) {
                this.removeContent();
                this.content = contentView;
                this.render();
            },

            select: function (gadget) {
                this.options.gadget = {
                    alias : gadget.alias
                };
                this.updateContentView();
            },

            updateContentView: function () {
                // Switch to *config-view*/*over-view* and
                // setup dialog buttons (Cancel and Apply, if config-view),
                // and dialog title.
                var gadget = this.options.gadget,
                    buttons = [],
                    // Pass context to require call. hence the use of *_.bind*.
                    initOverView = _.bind(function (OverView) {
                        var overView = new OverView();
                        this.renderWith(overView);
                        this.listenTo(this.content, 'select', this.select);
                        // Set title!!!! TODO
                        this.dialogNode.uiDialog('option', {
                            'buttons' : [this.cancelButton],
                            'title' : dict.translate('{InsertNewGadget}')
                        }).uiDialog('open');
                    }, this),
                    initConfigView = _.bind(function (ConfigView) {
                        var configView = new ConfigView(this.options.gadget.options);
                        // Set title!!!! TODO
                        this.renderWith(configView);
                        this.dialogNode.uiDialog('option', {
                            'buttons' : [this.applyButton, this.cancelButton],
                            'title' : dict.translate(this.options.gadget.alias)
                        }).uiDialog('open');
                    }, this);


                if (gadget) {
                    // Go to *config-view*.
                    require([modulePath + gadget.alias + '-view'], initConfigView);
                } else {
                    // Go to *over-view*.
                    require([modulePath + 'over' + '-view'], initOverView);
                }
            },

            initialize: function () {
                // We will need those later,
                // if we need to create the *over-view*.
                this.gadgetData = this.options;

                // Init dialog buttons.
                this.cancelButton = {
                    text: dict.translate('{Cancel}'),
                    click: $.proxy(this.close, this)
                };
                this.applyButton = {
                    text: dict.translate('{Apply}'),
                    click: $.proxy(this.apply, this)
                };

                // Init dialog.
                this.dialogNode = this.$el.uiDialog({
                    autoOpen: false,
                    stack: false,
                    // Quick fix to have colorpicker in contentslider view on top
                    // even after clicking somewhere else on the dialog.
                    width: 900,
                    height: 610,
                    modal: true,
                    title: dict.translate('{InsertNewGadget}')
                });
            }
        });

    return MainView;

});