package DE_EPAGES::ShopConfiguration::t::ShopExportCSV;
use base DE_EPAGES::Database::API::Test;

use Test::More tests => 8;
use Data::Dumper;
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile MoveFile );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::ShopConfiguration::API::ShopExportCSV qw( ExportShopStatusByProvider ExportShopStatusByStore ShopExportCSVConfig );
use strict;

my $CONFIG = "$ENV{'EPAGES_CONFIG'}/ShopExportCSV.conf";
my $CONFIG_BAK = "$ENV{'EPAGES_CONFIG'}/ShopExportCSV.conf.bak";
my $PROVIDER_CSV = 'Provider.csv';
my $STORE_CSV = 'Store.csv';

sub test {

    my $System = LoadRootObject();

    my $hConfig = ShopExportCSVConfig('Store');
    is_deeply( $hConfig, {
        'Conditions' => { 'IsClosed' => '0', 'IsTrialShop' => '0' },
        'Columns' => [ 'Alias', 'ShopType', 'Database', 'ObjectPath', 'IsClosed', 'IsTrialShop' ]
    }, 'ShopExportCSVConfig' );

    my $Store = $System->folder('Stores')->child('Store');
    ExportShopStatusByStore( $Store, $STORE_CSV, undef,  {
        Database => 'Store-Name',
        Provider => 'Papa'
    });
    my $aData = _ReadCSV( $STORE_CSV );
    cmp_ok( scalar @$aData, '>=', 1, 'At least one shop for store' );
    my $hShop = $aData->[0];
    is( scalar keys %$hShop, 15, 'don\t use columns specified in ShopExportCSV.conf' ) or diag Dumper($aData);
    is( $hShop->{Database}, 'Store-Name', 'common database name' ) or diag Dumper($hShop);
    is( $hShop->{Provider}, 'Papa', 'common provider name' ) or diag Dumper($hShop);

    my $Provider = $System->folder('Providers')->child('Distributor');
    my @Errors = ExportShopStatusByProvider( $Provider, $PROVIDER_CSV );
    is( scalar @Errors, 0, 'ExportShopStatusByProvider: no errors' ) or diag join "\n", @Errors;

    my $aData = _ReadCSV( $PROVIDER_CSV );
    cmp_ok( scalar @$aData, '>=', 1, 'At least one shop for provider' );
    $hShop = $aData->[0];
    is( scalar keys %$hShop, 6, 'using columns specified in ShopExportCSV.conf' ) or diag Dumper($aData);
}

sub init {

    die "backup file $CONFIG_BAK still exists" if ExistsFile( $CONFIG_BAK );
    MoveFile( $CONFIG, $CONFIG_BAK, { 'stat' => 'preserve' } )if ExistsFile( $CONFIG );

    my $Config = DE_EPAGES::Core::API::IniConfig->create( FileName => $CONFIG, CaseSensitive => 1 );
    $Config->section('Store', {
        Columns => 'Alias,ShopType,Database,ObjectPath,IsClosed,IsTrialShop',
        Conditions => 'IsClosed=0,IsTrialShop=0',
    });
    $Config->rewrite;
}

sub cleanup {

    MoveFile( $CONFIG_BAK, $CONFIG, { 'stat' => 'preserve' } ) if ExistsFile( $CONFIG_BAK );
    DeleteFile( $STORE_CSV );
    DeleteFile( $PROVIDER_CSV );
}

sub _ReadCSV {
    my ($FileName) = @_;

    my $CSV = DE_EPAGES::Core::API::CSV->new;
    my $fh = FileHandle->new;
    $fh->open( $FileName, '<:encoding(utf-8)' ) or die "Can't read $FileName: $!";
    my $aColumns = $CSV->getline($fh);
    my @Data;
    while( my $aValues = $CSV->getline($fh) ) {
        my %Row;
        @Row{@$aColumns} = @$aValues;
        push @Data, \%Row;
    }

    return \@Data;
}

__PACKAGE__->runScript;
